/*:ja

@plugindesc HPや状態異常などの時に顔グラを自動で変えます

@author Saimon

@param LefreshLv
@desc　常時描写するか？（メモなどで顔グラ呼んだ時にうまく映らない時に　2=常に　2以外=一度顔グラ変わったら5フレーム程度）
@default 1

@noteParam saimonFace_50_name
@noteRequire 1
@noteDir img/faces
@noteType file
@noteData actors

@noteParam saimonFace_20_name
@noteRequire 1
@noteDir img/faces
@noteType file
@noteData actors

@help
メニュー、アイテム、スキル、バトルのシーンで顔グラが変わります。
特定の状態異常は複数あると数の少ない数値が優先される

順番（上ほど優先）
死亡
特定の状態異常A
ダメージ受け
行動
特定の状態異常B
状態異常
HPが20%以下
HPが50%以下

アクターのメモ:
通常時はデフォルトの顔グラ番号
<saimonFace_50:顔グラ番号(0~7)>:HPが50%以下の時の顔グラ番号
<saimonFace_20:顔グラ番号(0~7)>:HPが20%以下の時の顔グラ番号
<saimonFace_died:顔グラ番号(0~7)>:死亡時の顔グラ番号
<saimonFace_action:顔グラ番号(0~7)>:行動時した時の顔グラ番号
<saimonFace_damege:顔グラ番号(0~7)>:ダメージ受けた時の顔グラ番号
<saimonFace_state:顔グラ番号(0~7)>:状態異常時の顔グラ番号
<saimonFace_specificA数値(0~9):顔グラ番号(0~7)>:特定の状態異常A時の顔グラ番号
<saimonFace_specificB数値(0~9):顔グラ番号(0~7)>:特定の状態異常B時の顔グラ番号

設定しなければデフォルトの顔グラ名
<saimonFace_50_name:顔グラ名>:HPが50%以下の時の顔グラ名
<saimonFace_20_name:顔グラ名>:HPが20%以下の時の顔グラ名
<saimonFace_died_name:顔グラ名>:死亡時の顔グラ名
<saimonFace_action_name:顔グラ名>:行動時した時の顔グラ名
<saimonFace_damege_name:顔グラ名>:ダメージ受けた時の顔グラ名
<saimonFace_state_name:顔グラ名>:状態異常時の顔グラ名
<saimonFace_specificA数値(0~9)_name:顔グラ名>:特定の状態異常A時の顔グラ名
<saimonFace_specificB数値(0~9)_name:顔グラ名>:特定の状態異常B時の顔グラ名


ステートのメモ:
<saimonFace_states:bad>:状態異常時の顔グラ（毒、暗闇などの悪い状態異常）
<saimonFace_statesA:badA数値(0~9)>:状態異常A時の顔グラ（石化などの顔グラ変わらない状態異常）
<saimonFace_statesB:badB数値(0~9)>:状態異常B時の顔グラ


改造関数（プラグインの競合注意）:
BattleManager.startAction

Game_Battler.prototype.initMembers

Scene_ItemBase.prototype.update
Scene_Menu.prototype.update
Scene_Skill.prototype.update
Scene_Battle.prototype.update

Window_BattleLog.prototype.endAction
Window_BattleLog.prototype.makeHpDamageText


*/

(function() {
    
    var parameters = PluginManager.parameters('SaimonFace');//受けとるためのまじない？
    var lefreshLv = String(parameters['LefreshLv']);
    
    var loop = 0;
    
    //行動時
    BattleManager.startAction = function() {
        var subject = this._subject;
        
        subject._saimonface = "action";//行動時
        
        var action = subject.currentAction();
        var targets = action.makeTargets();
        this._phase = 'action';
        this._action = action;
        this._targets = targets;
        subject.useItem(action.item());
        this._action.applyGlobal();
        this.refreshStatus();
        this._logWindow.startAction(subject, action, targets);
    };
    
    //アクターとエネミーの中設定
    Game_Battler.prototype.initMembers = function() {
        Game_BattlerBase.prototype.initMembers.call(this);
        this._actions = [];
        this._speed = 0;
        this._result = new Game_ActionResult();
        this._actionState = null;
        this._lastTargetIndex = 0;
        this._animations = [];
        this._damagePopup = false;
        this._effectType = null;
        this._motionType = null;
        this._weaponImageId = 0;
        this._motionRefresh = false;
        this._selected = false;

        this._saimonface = null;//攻撃時などの判定
    };
    
    //行動終了時
    Window_BattleLog.prototype.endAction = function(subject) {
        this.push('waitForNewLine');
        this.push('clear');
        this.push('performActionEnd', subject);
        
        var actor = $gameParty.members();//パーティーメンバー取得
        var enemie = $gameTroop._enemies;//敵取得
        for(var r = 0;r < actor.length;r++){
            actor[r]._saimonface = null;//リセット
        }
        for(var r = 0;r < enemie.length;r++){
            enemie[r]._saimonface = null;//リセット
        }
    };
    
    //ダメージ受けた時
    Window_BattleLog.prototype.makeHpDamageText = function(target) {
        var result = target.result();
        var damage = result.hpDamage;
        var isActor = target.isActor();
        var fmt;
        if (damage > 0 && result.drain) {
            target._saimonface = "damage";//ダメージ受ける
            SaimonFaceChange();//いろいろ変える
            
            fmt = isActor ? TextManager.actorDrain : TextManager.enemyDrain;
            return fmt.format(target.name(), TextManager.hp, damage);
        } else if (damage > 0) {
            target._saimonface = "damage";//ダメージ受ける
            SaimonFaceChange();//いろいろ変える
            
            fmt = isActor ? TextManager.actorDamage : TextManager.enemyDamage;
            return fmt.format(target.name(), damage);
        } else if (damage < 0) {
            fmt = isActor ? TextManager.actorRecovery : TextManager.enemyRecovery;
            return fmt.format(target.name(), TextManager.hp, -damage);
        } else {
            fmt = isActor ? TextManager.actorNoDamage : TextManager.enemyNoDamage;
            return fmt.format(target.name());
        }
    };
    
    //アイテムメニューウィンドウ常
    Scene_ItemBase.prototype.update = function() {
        SaimonFaceChange();//いろいろ変える
        if(loop > 0 || lefreshLv == 2){
            this._actorWindow.refresh();//もう一度描写
            loop--;
        }    
        
        this.updateFade();
        this.updateChildren();
    };
    
    //メニューウィンドウ常
    Scene_Menu.prototype.update = function() {
        SaimonFaceChange();//いろいろ変える
        if(loop > 0 || lefreshLv == 2){
            this._statusWindow.refresh();;//もう一度描写
            loop--;
        }    
        
        this.updateFade();
        this.updateChildren();
    };
     
    //スキルメニューウィンドウ常
    Scene_Skill.prototype.update = function() {
    SaimonFaceChange();//いろいろ変える
        if(loop > 0 || lefreshLv == 2){
            this._statusWindow.refresh();//もう一度描写
            this._actorWindow.refresh();//もう一度描写
            loop--;
        }    
        
        this.updateFade();
        this.updateChildren();
};
    
    //バトルウィンドウ常
    Scene_Battle.prototype.update = function() {
    SaimonFaceChange();//いろいろ変える
    if(loop > 0 || lefreshLv == 2){
        this._statusWindow.refresh();//もう一度描写
        loop--;
    }    
        
    var active = this.isActive();
    $gameTimer.update(active);
    $gameScreen.update();
    this.updateStatusWindow();
    this.updateWindowPositions();
    if (active && !this.isBusy()) {
        this.updateBattleProcess();
    }
    Scene_Base.prototype.update.call(this);
};
    
    //顔グラ変える 　自作関数
    SaimonFaceChange = function(){
        var actor = $gameParty.members();//パーティーメンバー取得
        for(var r = 0;r < actor.length;r++){
            var id = actor[r]._actorId;//ID
            var faceName = null;//顔グラ名
            var mhp = actor[r].mhp;//MaxHP
            var hp = actor[r]._hp;//HP
            var states = actor[r]._states;//状態異常
            var damage = actor[r]._damagePopup;//ダメージ受けてるか？
            var action = actor[r]._saimonface;//アクション
            var dateFaceName = $dataActors[id].faceName;//デフォルト顔グラ
            var dateFaceIndex = $dataActors[id].faceIndex;//デフォルト顔グラ番号
            var meta = $dataActors[id].meta;//アクターのメタデータ取得
                        var judgmentFaseName = actor[r]._faceName;//今の顔グラ名
            var judgmentFaseindex = actor[r]._faceIndex;//今の顔グラ番号
            
            /*順番（上ほど優先）
            死亡
            特定の状態異常A
            ダメージ受け
            行動
            特定の状態異常B
            状態異常
            HPが20%以下
            HPが50%以下*/
            
            //死亡
            if(hp == 0 && meta.saimonFace_died != null){
                faceName = (meta.saimonFace_died_name != null) ? meta.saimonFace_died_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_died);//顔グラ変更
                
            //特定の状態異常A
            }else if(StatesCheck(states,"badA0") && meta.saimonFace_specificA0 != null){
                faceName = (meta.saimonFace_specificA0_name != null) ? meta.saimonFace_specificA0_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA0);//顔グラ変更
            }else if(StatesCheck(states,"badA1") && meta.saimonFace_specificA1 != null){
                faceName = (meta.saimonFace_specificA1_name != null) ? meta.saimonFace_specificA1_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA1);//顔グラ変更
            }else if(StatesCheck(states,"badA2") && meta.saimonFace_specificA2 != null){
                faceName = (meta.saimonFace_specificA2_name != null) ? meta.saimonFace_specificA2_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA2);//顔グラ変更
            }else if(StatesCheck(states,"badA3") && meta.saimonFace_specificA3 != null){
                faceName = (meta.saimonFace_specificA3_name != null) ? meta.saimonFace_specificA3_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA3);//顔グラ変更
            }else if(StatesCheck(states,"badA4") && meta.saimonFace_specificA4 != null){
                faceName = (meta.saimonFace_specificA4_name != null) ? meta.saimonFace_specificA4_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA4);//顔グラ変更
            }else if(StatesCheck(states,"badA5") && meta.saimonFace_specificA5 != null){
                faceName = (meta.saimonFace_specificA5_name != null) ? meta.saimonFace_specificA5_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA5);//顔グラ変更
            }else if(StatesCheck(states,"badA6") && meta.saimonFace_specificA6 != null){
                faceName = (meta.saimonFace_specificA6_name != null) ? meta.saimonFace_specificA6_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA6);//顔グラ変更
            }else if(StatesCheck(states,"badA7") && meta.saimonFace_specificA7 != null){
                faceName = (meta.saimonFace_specificA7_name != null) ? meta.saimonFace_specificA7_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA7);//顔グラ変更
            }else if(StatesCheck(states,"badA8") && meta.saimonFace_specificA8 != null){
                faceName = (meta.saimonFace_specificA8_name != null) ? meta.saimonFace_specificA8_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA8);//顔グラ変更
            }else if(StatesCheck(states,"badA9") && meta.saimonFace_specificA9 != null){
                faceName = (meta.saimonFace_specificA9_name != null) ? meta.saimonFace_specificA9_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificA9);//顔グラ変更
                
            //ダメージ受け
            }else if(action == "damage" && meta.saimonFace_damege != null){
                faceName = (meta.saimonFace_damege_name != null) ? meta.saimonFace_damege_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_damege);//顔グラ変更
            //行動    
            }else if(action == "action" && meta.saimonFace_action != null){
                faceName = (meta.saimonFace_action_name != null) ? meta.saimonFace_action_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_action);//顔グラ変更
                
            //特定の状態異常B
            }else if(StatesCheck(states,"badB0") && meta.saimonFace_specificB0 != null){
                faceName = (meta.saimonFace_specificB0_name != null) ? meta.saimonFace_specificB0_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB0);//顔グラ変更
            }else if(StatesCheck(states,"badB1") && meta.saimonFace_specificB1 != null){
                faceName = (meta.saimonFace_specificB1_name != null) ? meta.saimonFace_specificB1_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB1);//顔グラ変更
            }else if(StatesCheck(states,"badB2") && meta.saimonFace_specificB2 != null){
                faceName = (meta.saimonFace_specificB2_name != null) ? meta.saimonFace_specificB2_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB2);//顔グラ変更
            }else if(StatesCheck(states,"badB3") && meta.saimonFace_specificB3 != null){
                faceName = (meta.saimonFace_specificB3_name != null) ? meta.saimonFace_specificB3_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB3);//顔グラ変更
            }else if(StatesCheck(states,"badB4") && meta.saimonFace_specificB4 != null){
                faceName = (meta.saimonFace_specificB4_name != null) ? meta.saimonFace_specificB4_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB4);//顔グラ変更
            }else if(StatesCheck(states,"badB5") && meta.saimonFace_specificB5 != null){
                faceName = (meta.saimonFace_specificB5_name != null) ? meta.saimonFace_specificB5_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB5);//顔グラ変更
            }else if(StatesCheck(states,"badB6") && meta.saimonFace_specificB6 != null){
                faceName = (meta.saimonFace_specificB6_name != null) ? meta.saimonFace_specificB6_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB6);//顔グラ変更
            }else if(StatesCheck(states,"badB7") && meta.saimonFace_specificB7 != null){
                faceName = (meta.saimonFace_specificB7_name != null) ? meta.saimonFace_specificB7_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB7);//顔グラ変更
            }else if(StatesCheck(states,"badB8") && meta.saimonFace_specificB8 != null){
                faceName = (meta.saimonFace_specificB8_name != null) ? meta.saimonFace_specificB8_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB8);//顔グラ変更
            }else if(StatesCheck(states,"badB9") && meta.saimonFace_specificB9 != null){
                faceName = (meta.saimonFace_specificB9_name != null) ? meta.saimonFace_specificB9_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_specificB9);//顔グラ変更
                
            //状態異常
            }else if(StatesCheck(states,"bad") && meta.saimonFace_state != null){
                faceName = (meta.saimonFace_state_name != null) ? meta.saimonFace_state_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_state);//顔グラ変更
            //HPが20%以下
            }else if(hp < mhp / 100 * 25 && meta.saimonFace_20 != null){
                faceName = (meta.saimonFace_20_name != null) ? meta.saimonFace_20_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_20);//顔グラ変更
            //HPが50%以下
            }else if(hp < mhp / 2 && meta.saimonFace_50 != null){
                faceName = (meta.saimonFace_50_name != null) ? meta.saimonFace_50_name:dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, meta.saimonFace_50);//顔グラ変更
            //通常
            }else{
                faceName = dateFaceName;//顔グラ名指定
                $gameActors.actor(id).setFaceImage(faceName, dateFaceIndex);//顔グラ変更
            }
            
            //画像変えたら数回描写（メモ指定のの顔グラがうまく表示されない関係で・・・）
            if(judgmentFaseName != faceName || judgmentFaseindex != actor[r]._faceIndex){
                loop = 5;
            }
        }
    }
        
    //状態異常調べる　自作関数
    StatesCheck = function (states,word){//アクターの状態異常,調べる言葉
        var check = false;
        for(var r = 0;r < states.length;r++){
            var id = states[r];//ID
            var meta = $dataStates[id].meta;//メタデータ取得
            if(meta.saimonFace_states == word || 
               meta.saimonFace_statesA == word || 
               meta.saimonFace_statesB == word){
                check = true;
                break;
            }
        }
        return check;
    }
        

})();